/*
 * @Cameron Giarrusso Game Apr 6, 2017
 */
package Monopoly;

import Monopoly.Board.Board;
import Monopoly.Board.Space.Special;
import static Monopoly.Board.Space.Special.GO_TO_JAIL;
import Monopoly.Cards.Cards;
import static Monopoly.Cards.ChanceCard.CHANCE_ADVANCE_TOKEN_TO_NEAREST_RAILROAD;
import static Monopoly.Cards.ChanceCard.CHANCE_ADVANCE_TOKEN_TO_NEAREST_UTILITY;
import static Monopoly.Cards.ChanceCard.CHANCE_GO_DIRECTLY_TO_JAIL;
import static Monopoly.Cards.CommunityChestCard.DOCTOR_FEE;
import static Monopoly.Cards.CommunityChestCard.GO_DIRECTLY_TO_JAIL;
import Monopoly.Rolls.Roll;

/**
 *
 * @author fcg
 */

//This class runs each game of monopoly
public class Game {

    //These are all the variables needed to store information about a game of monopoly
    private int rolls = 0;
    private int moves = 0;
    private int position = 0;
    private int pastGo = 0;
    private int distance = 0;
    private int doubles = 0;
    private int doublesInARow = 0;
    private int jailsInARow = 0;
    private int doubleDoubles = 0;
    private int doublesToJail = 0;
    private int rollsInJail = 0;
    private int doublesInJail = 0;
    private int drawCard = 0;
    private int cardMove = 0;
    private boolean inJail = false;
    private Board board;
    private Roll roll;
    private Cards card;
    private boolean useCards;

    //This constructor establishes whether the user wants to use cards
    public Game(boolean useCards) {
        this.board = new Board();
        this.roll = new Roll();
        this.useCards = useCards;
    }

    /**
     * This method conducts the entire turn for the player If the player is in
     * Jail, instead of proceeding with the turn it calls the jail method If the
     * player rolls 3 doubles
     */
    public void turn() {
        
        //Checks if the user is in jail, if so then the program switches to the jail method
        if (inJail) {
            unjail();
            return;
        }
        
        //Rolls the dice
        distance += roll.roll();
        rolls++;
        
        //Checks if the roll is a doubles and stores the information
        if (roll.isDoubles()) {
            doubles++;
            doublesInARow++;
            if (doublesInARow == 2) {
                doubleDoubles++;
            }
            //Checks to see if the user had rolled 3 doubles in a row, in which case they are sent to jail
            if (doublesInARow >= 3) {
                doublesInARow = 0;
                doublesToJail++;
                jail();
                return;
            }
            move();
        } else {
            move();
            doublesInARow = 0;
        }
    }

    //This method moves the player token
    public void move() {
        moves++;
        position += roll.getRoll();
        
        //If the player passes go, the position index is reset
        if (position >= 40) {
            pastGo++;
            position %= 40;
        }
        
        //Stores the information for the number of times a spaces was landed on
        board.getSpace(position).addLanded();
        if (useCards == true) {
            checkSpot();
        }
        if (board.getSpace(position) == GO_TO_JAIL) {
            jail();
        }
    }

    //Sets a flag so the player knows they are in jail
    public void jail() {
        inJail = true;
        jailsInARow = 0;
        position = 10;
    }

    //Tries to roll doubles to get a player out of jail
    public void unjail() {
        jailsInARow++;
        rollsInJail++;
        roll.roll();
        rolls++;
        
        //Checks of the roll is a doubles
        if (roll.isDoubles()) {
            doublesInJail++;
            distance += roll.getRoll();
            move();
            inJail = false;
        } 
        
        //Checks if this is the players third roll in jail, if so then the player is released
        else if (jailsInARow >= 3) {
            distance += roll.getRoll();
            move();
            inJail = false;
        }
    }

    //Checks the spot the player is currently on to check if a card needs to be drawn
    public void checkSpot() {
        Special[] chanceCards = board.getChances();
        Special[] communityChestCards = board.getCommunityChests();
        for (Special i : chanceCards) {
            if (i.getSpace() == position) {
                chance();
            }
        }
        for (Special i : communityChestCards) {
            if (i.getSpace() == position) {
                chest();
            }
        }
    }

    //If a chance card is drawn then this method completes its action
    public void chance() {
        drawCard++;
        card = CHANCE_GO_DIRECTLY_TO_JAIL.drawCard();
            if (card.getSpace() > -1) {
                position = card.getSpace();
                board.getSpace(position).addLanded();
                cardMove++;
            } else if (card == CHANCE_ADVANCE_TOKEN_TO_NEAREST_RAILROAD) {
                position = board.getNearestRailroad(position).getSpace();
                board.getSpace(position).addLanded();
                cardMove++;
            } else if (card == CHANCE_ADVANCE_TOKEN_TO_NEAREST_UTILITY) {
                position = board.getNearestUtility(position).getSpace();
                board.getSpace(position).addLanded();
                cardMove++;
            }
            else if (card == GO_DIRECTLY_TO_JAIL) {
                jail();
                cardMove++;
            }
            else if (card.getMove() != 0) {
            position += card.getMove();
            cardMove++;
        }
            return;
    }

    //If a community chest card is drawn then this method completes its action
    public void chest() {
        drawCard++;
        card = DOCTOR_FEE.drawCard();
        if (card.getSpace() > -1) {
            position = card.getSpace();
            board.getSpace(position).addLanded();
            if (card == GO_DIRECTLY_TO_JAIL) {
                jail();
            }
            cardMove++;
            return;
        }
        if (card.getMove() != 0) {
            position += card.getMove();
            cardMove++;
        }
    }

    //This method resets the players position based on where they started the game at
    public void reset(int position) {
        this.position = position;
        inJail = false;
    }

    public Special getNearestUtility() {
        return board.getNearestUtility(position);
    }

    public Special getNearestRailroad() {
        return board.getNearestRailroad(position);
    }

    public int getRolls() {
        return rolls;
    }

    public int getMoves() {
        return moves;
    }

    public int getPosition() {
        return position;
    }

    public int getDistance() {
        return distance;
    }

    public int getDoubles() {
        return doubles;
    }

    public int getDoublesInARow() {
        return doublesInARow;
    }

    public int getJailsInARow() {
        return jailsInARow;
    }

    public int getRollsInJail() {
        return rollsInJail;
    }

    public boolean getInJail() {
        return inJail;
    }

    public Board getBoard() {
        return board;
    }

    public Roll getRoll() {
        return roll;
    }

    public int getPastGo() {
        return pastGo;
    }

    public int getDoublesToJail() {
        return doublesToJail;
    }

    public int getDoubleDoubles() {
        return doubleDoubles;
    }

    public int getDoublesInJail() {
        return doublesInJail;
    }

    public int getCardDraws() {
        return drawCard;
    }

    public int getCardMoves() {
        return cardMove;
    }

}
